use ros_message::I8Variant;

use crate::SmallStr;

#[derive(Clone, Debug)]
pub enum FieldType {
    /// Represents `bool`.
    Bool,
    /// Represents `int8` or `byte`.
    I8(I8Variant),
    /// Represents `int16`.
    I16,
    /// Represents `int32`.
    I32,
    /// Represents `int64`.
    I64,
    /// Represents `uint8` or `char`.
    U8(I8Variant),
    /// Represents `uint16`.
    U16,
    /// Represents `uint32`.
    U32,
    /// Represents `uint64`.
    U64,
    /// Represents `float32`.
    F32,
    /// Represents `float64`.
    F64,
    /// Represents `string`.
    String,
    /// Represents `time`.
    Time,
    /// Represents `duration`.
    Duration,
    // Another messgage type.
    Message(MessageLayoutRef),
}

#[derive(Clone, Debug)]
pub struct FieldLayout {
    pub typ: FieldType,
    pub name: SmallStr,
    pub multiplicity: Multiplicity,
}

#[derive(Clone, Debug, PartialEq, Eq)]
pub enum Multiplicity {
	Unit,
	Fixed(usize),
	Dynamic,
}

#[derive(Clone, Debug)]
pub struct MessageLayout {
    fields: Vec<FieldLayout>,
}

pub type MessageLayoutRef = std::sync::Arc<MessageLayout>;
